@@ -39,6 +39,7 @@
 #include <gconf/gconf-client.h>
 
 #include "hildon-home-image-loader.h"
+#include <osso-log.h>
 
 /*SAW (allocation watchdog facilities)*/
 #include <osso-mem.h>
@@ -100,7 +101,8 @@ load_image_oom_cb(size_t current_sz, size_t max_sz,void *context)
  * Load an image from given URI. Places to pixbuf
 */
 static 
-gint load_image_from_uri(GdkPixbuf **pixbuf, const gchar *uri, gboolean postinstall)
+gint load_image_from_uri(GdkPixbuf **pixbuf, const gchar *uri, 
+                         gboolean postinstall)
 {     
     GError           *error = NULL;
     GdkPixbufLoader  *loader;
@@ -143,21 +145,41 @@ gint load_image_from_uri(GdkPixbuf **pixbuf, const gchar *uri, gboolean postinst
         } 	 
     }
     g_free(mmc_uri_prefix);
+    
     result = gnome_vfs_open(&handle, uri, GNOME_VFS_OPEN_READ);
+    
+    if(result != GNOME_VFS_OK)
+    {
+        g_object_unref(gconf_client);
+        return HILDON_HOME_IMAGE_LOADER_ERROR_FILE_UNREADABLE;
+    }
+
     /*Setting up a watchdog*/
-    if(!postinstall && osso_mem_saw_enable( 3 << 20, 32767, load_image_oom_cb, (void *)&oom) )
+    if(!postinstall && 
+       osso_mem_saw_enable( 3 << 20, 32767, load_image_oom_cb, (void *)&oom) )
     {
         oom = TRUE;
     } 
     loader = gdk_pixbuf_loader_new();
 
-    while (!oom && (result == GNOME_VFS_OK) && (!mmc_in_use || !mmc_cover_open)) {
+    while (!oom && (result == GNOME_VFS_OK) && 
+           (!mmc_in_use || !mmc_cover_open)) {
         result = gnome_vfs_read(handle, buffer, BUF_SIZE, &bytes_read);
         if(!oom) 
         {
-            if(!gdk_pixbuf_loader_write(loader, buffer, bytes_read, NULL))
+            if(!gdk_pixbuf_loader_write(loader, buffer, bytes_read, &error))
             {
-                return HILDON_HOME_IMAGE_LOADER_ERROR_SYSTEM_RESOURCE; 	 
+                if(error != NULL)
+                {
+                    if(error->domain == GDK_PIXBUF_ERROR &&
+                            (error->code == GDK_PIXBUF_ERROR_CORRUPT_IMAGE ||
+                             error->code == GDK_PIXBUF_ERROR_UNKNOWN_TYPE))
+                    {
+                        g_error_free(error);
+                        return HILDON_HOME_IMAGE_LOADER_ERROR_FILE_CORRUPT;
+                    }
+                    g_error_free(error);
+                }
             }
         }
 
@@ -202,6 +224,8 @@ gint load_image_from_uri(GdkPixbuf **pixbuf, const gchar *uri, gboolean postinst
     gdk_pixbuf_loader_close(loader, NULL);
     *pixbuf = gdk_pixbuf_loader_get_pixbuf(loader);
     gnome_vfs_close(handle);
+    if (!*pixbuf)
+        return HILDON_HOME_IMAGE_LOADER_ERROR_FILE_CORRUPT;
     return HILDON_HOME_IMAGE_LOADER_OK;
 }
 
@@ -264,6 +288,15 @@ gint save_image(GdkPixbuf *pixbuf, const gchar *path)
     if (!gdk_pixbuf_save(pixbuf, temp_path, 
                          HILDON_HOME_IMAGE_FORMAT, &error, NULL)) 
     {
+        if (error && error->domain == G_FILE_ERROR 
+                  && error->code == G_FILE_ERROR_NOSPC)
+        {
+            /* Flash full! */
+            g_error_free(error);
+            unlink(temp_path);
+            g_free(temp_path);
+            return HILDON_HOME_IMAGE_LOADER_ERROR_FLASH_FULL;
+        }
         g_error_free(error);
         unlink(temp_path);
         g_free(temp_path);
@@ -279,6 +312,41 @@ gint save_image(GdkPixbuf *pixbuf, const gchar *path)
     return HILDON_HOME_IMAGE_LOADER_OK;
 }
 
+/**
+ * @create_background_pixbuf_color
+ *
+ * @param pixbuf Storing place of created pixbuf
+ * @param width Width of the destination pixbuf to create
+ * @param height Height of the destination pixbuf to create
+ * @param color_red Red component of rgb color to set background of pixbuf
+ * @param color_green Green component of rgb color to set background of pixbuf
+ * @param color_blue Blue component of rgb color to set background of pixbuf
+ *
+ * @returns loader status
+ *
+ * Create a background image of given rgb color.
+ */
+
+static
+void create_background_pixbuf_color(GdkPixbuf **pixbuf, 
+                                    gint width, gint height,
+                                    gint color_red, 
+                                    gint color_green, 
+                                    gint color_blue)
+{
+    guint32 color = 0;
+   
+    color = (guint8)(color_red >> 8) << 24 | 
+            (guint8)(color_green >> 8) << 16 |
+            (guint8)(color_blue >> 8) << 8 |
+            0xff;
+
+    *pixbuf = gdk_pixbuf_new(GDK_COLORSPACE_RGB, TRUE, 8, 
+                             width, height);
+    
+    gdk_pixbuf_fill(*pixbuf, color);
+}
+
 /**
  * @create_background_pixbuf
  *
@@ -289,101 +357,130 @@ gint save_image(GdkPixbuf *pixbuf, const gchar *path)
  *
  * @returns loader status
  *
- * Create a background from given source image. If the image is larger than
- * the destination size, it's cropped. If it's smaller, it's tiled to fill space.
+ * Create a background from given source image. If the image is larger
+ * than the destination size, it's cropped. If it's smaller, it's
+ * tiled to fill space.
  */
 
 static
 gint create_background_pixbuf(GdkPixbuf **pixbuf, const GdkPixbuf *src_pixbuf,
-                              gint width, gint height)
+                              gint width, gint height, 
+                              gint scaling,
+                              gint color_red, 
+                              gint color_green, 
+                              gint color_blue)
 {
     gint src_width = -1, src_height = -1;
-    gint part_width = -1, part_height = -1;
-    gint src_x, src_y, dest_x, dest_y;
-    GdkPixbuf *part_pixbuf = NULL;
-    
+    gint dest_x, dest_y;
+    double scaling_ratio;
+
     if (src_pixbuf == NULL) 
     {
         return HILDON_HOME_IMAGE_LOADER_ERROR_SYSTEM_RESOURCE;        
     }
 
+
+    create_background_pixbuf_color(pixbuf, 
+                                   width, height,
+                                   color_red, color_green, color_blue);
+    if (*pixbuf == NULL) 
+    {
+        return HILDON_HOME_IMAGE_LOADER_ERROR_SYSTEM_RESOURCE;
+    }
+
     src_width = gdk_pixbuf_get_width(src_pixbuf);
     src_height = gdk_pixbuf_get_height(src_pixbuf);
-    
+
     if (src_width == width && src_height == height) 
     {
         /* source pixbuf is in the exactly wanted size,
            we can use it directly */
-        *pixbuf = gdk_pixbuf_copy(src_pixbuf);
-        return HILDON_HOME_IMAGE_LOADER_OK; 
-    }
+        gdk_pixbuf_composite(src_pixbuf,
+                             *pixbuf,
+                             0, 0,
+                             width, height,
+                             0, 0,
+                             1.0, 1.0,
+                             GDK_INTERP_NEAREST,
+                             HILDON_HOME_IMAGE_ALPHA_FULL);
 
-    /* if image is larger than wanted, crop from the middle */
-    if(src_width > width || src_height > height)
-    {
-        part_width = MIN(src_width, width);
-        part_height = MIN(src_height, height);
-
-        part_pixbuf = gdk_pixbuf_new(GDK_COLORSPACE_RGB, TRUE, 8, 
-                                     part_width, part_height);
-
-        if (part_pixbuf == NULL) 
+        if (*pixbuf == NULL) 
         {
             return HILDON_HOME_IMAGE_LOADER_ERROR_SYSTEM_RESOURCE;
         }
+        return HILDON_HOME_IMAGE_LOADER_OK; 
+    }
 
-        src_x = src_width > width ? (src_width - width) / 2 : 0;
-        src_y = src_height > height ? (src_height - height) / 2 : 0;
+    switch (scaling)
+    {
+      case CENTERED:
+        /* desk_x/y and offset_x/y behave diffrently when source image
+           is smaller than when its bigger
+         */
+        gdk_pixbuf_composite(src_pixbuf,
+                             *pixbuf,
+                             MAX(0, (width-src_width) / 2),
+                             MAX(0, (height-src_height) / 2),
+                             MIN(src_width, width),
+                             MIN(src_height, height),
+                             (width-src_width) / 2,
+                             (height-src_height) / 2,
+                             1.0, 1.0,
+                             GDK_INTERP_NEAREST,
+                             HILDON_HOME_IMAGE_ALPHA_FULL);
+        break;
+      case SCALED:
+        scaling_ratio = MIN((double)width/src_width, 
+                            (double)height/src_height);
 
-        gdk_pixbuf_copy_area(src_pixbuf, src_x, src_y,
-                             part_width,
-                             part_height,
-                             part_pixbuf, 0, 0);
+        gdk_pixbuf_composite(src_pixbuf,
+                             *pixbuf,
+                             (int)MAX(0,(width-scaling_ratio*src_width)/2),
+                             (int)MAX(0,(height-scaling_ratio*src_height)/2),
+                             scaling_ratio*src_width, 
+                             scaling_ratio*src_height,
+                             MAX(0,(width-scaling_ratio*src_width)/2),
+                             MAX(0,(height-scaling_ratio*src_height)/2),
+                             scaling_ratio,
+                             scaling_ratio,
+                             GDK_INTERP_NEAREST,
+                             HILDON_HOME_IMAGE_ALPHA_FULL);
 
-        
-        if (part_width == width && part_height == height) 
+        break;
+      case STRETCHED:
+        gdk_pixbuf_composite(src_pixbuf,
+                             *pixbuf,
+                             0, 0,
+                             width, height,
+                             0, 0,
+                             (double)width/src_width,
+                             (double)height/src_height,
+                             GDK_INTERP_NEAREST,
+                             HILDON_HOME_IMAGE_ALPHA_FULL);
+        break;
+      case TILED:
+
+        for(dest_x = 0; dest_x < width; dest_x += src_width)
         {
-            /* cropped pixbuf is in the exactly wanted size,
-               we can use it directly */
-            *pixbuf = gdk_pixbuf_copy(part_pixbuf);
-            if (part_pixbuf != NULL) 
+            for(dest_y = 0; dest_y < height; dest_y += src_height )
             {
-                g_object_unref(part_pixbuf);
+                gdk_pixbuf_composite(src_pixbuf,
+                                     *pixbuf, 
+                                     dest_x, 
+                                     dest_y,
+                                     MIN(src_width, width-dest_x),
+                                     MIN(src_height, height-dest_y),
+                                     dest_x, dest_y,
+                                     1.0, 1.0,
+                                     GDK_INTERP_NEAREST,
+                                     HILDON_HOME_IMAGE_ALPHA_FULL);
             }
-            return HILDON_HOME_IMAGE_LOADER_OK; 
-        }
-    } else {
-        part_width = src_width;
-        part_height = src_height;
-    }
-
-    /* image is smaller than wanted, fill image area by tiling it. */
-    *pixbuf = gdk_pixbuf_new(GDK_COLORSPACE_RGB, TRUE, 8, width, height);
-        
-    if (*pixbuf == NULL) 
-    {
-        if (part_pixbuf != NULL) 
-        {
-            g_object_unref(part_pixbuf);
-        }
-        return HILDON_HOME_IMAGE_LOADER_ERROR_SYSTEM_RESOURCE;
-    }
-        
-    for(dest_x = 0; dest_x < width; dest_x += part_width)
-    {
-        for(dest_y = 0; dest_y < height; dest_y += part_height )
-        {
-            gdk_pixbuf_copy_area(src_pixbuf, 0, 0,
-                                 MIN(part_width, width-dest_x),
-                                 MIN(part_height, height-dest_y),
-                                 *pixbuf, dest_x, dest_y);
         }
+        break;
+      default :
+        return HILDON_HOME_IMAGE_LOADER_ERROR_SYSTEM_RESOURCE;        
     }
     
-    if (part_pixbuf != NULL) 
-    {
-        g_object_unref(part_pixbuf);
-    }
     return HILDON_HOME_IMAGE_LOADER_OK;
 }
 
@@ -478,8 +575,7 @@ void blend_new_skin_to_image_area(GdkPixbuf **pixbuf_orig,
             HILDON_HOME_SIDEBAR_HEIGHT_LOADER,
             GDK_INTERP_NEAREST);                
         g_object_unref(pixbuf_blend);
-        pixbuf_blend = gdk_pixbuf_copy(pixbuf_temp);
-        g_object_unref(pixbuf_temp);
+        pixbuf_blend = pixbuf_temp;
         width_blend = HILDON_HOME_SIDEBAR_WIDTH_LOADER;
         height_blend = HILDON_HOME_SIDEBAR_HEIGHT_LOADER;
     }
@@ -490,7 +586,7 @@ void blend_new_skin_to_image_area(GdkPixbuf **pixbuf_orig,
                          height_blend,
                          0, 0,
                          1.0, 1.0,
-                         GDK_INTERP_BILINEAR,
+                         GDK_INTERP_NEAREST,
                          HILDON_HOME_IMAGE_ALPHA_FULL);
 
     g_object_unref(pixbuf_blend);
@@ -538,24 +634,26 @@ void blend_in_with_pixbuf(GdkPixbuf **pixbuf, GdkPixbuf *blended_pixbuf,
  * @blend_in_from_path
  *
  * @param pixbuf Storing place of blended image pixbuf
- * @param path Path for the image to read
+ * @param load_path Path for the image to read
  * @param save_path Path for the original image area to save
  * @param x Destination for X coordinate
  * @param y Destination for Y coordinate
+ * @param area_width Destination area width
+ * @param area_height Destination area height
  *
  * Read an image from given path and blend it into destination pixbuf
  * in the given location.
  */
 static 
 void blend_in_from_path(GdkPixbuf **pixbuf, 
-                        const gchar *path, 
+                        const gchar *load_path, 
                         const gchar *save_path, 
-                        gint x, gint y)
+                        gint area_x, gint area_y,
+                        gint area_width, gint area_height)
 {
     GdkPixbuf *src_pixbuf;
     GdkPixbuf *src_pixbuf_orig = NULL;
     gint width, height;
-    gint src_height, src_width;
     gint loader_status;
 
     if (*pixbuf == NULL) 
@@ -566,26 +664,23 @@ void blend_in_from_path(GdkPixbuf **pixbuf,
     width = gdk_pixbuf_get_width(*pixbuf);
     height = gdk_pixbuf_get_height(*pixbuf);
 
-    if (x > width || y > height) 
+    if (area_x > width || area_y > height) 
     {
         return;
     }
     
-    loader_status = load_image_pixbuf(&src_pixbuf, path);
+    loader_status = load_image_pixbuf(&src_pixbuf, load_path);
     if(loader_status != HILDON_HOME_IMAGE_LOADER_OK) 
     {
         return;
     }
 
-    src_width = gdk_pixbuf_get_width(src_pixbuf);
-    src_height = gdk_pixbuf_get_height(src_pixbuf);
-
     loader_status = 
         create_original_pixbuf_area(&src_pixbuf_orig,
                                     *pixbuf,
-                                    0, 0,
-                                    src_width,
-                                    src_height);
+                                    area_x, area_y,
+                                    area_width,
+                                    area_height);
     if(loader_status != HILDON_HOME_IMAGE_LOADER_OK) 
     {
         return;
@@ -599,112 +694,12 @@ void blend_in_from_path(GdkPixbuf **pixbuf,
     }
 
     gdk_pixbuf_composite(src_pixbuf, *pixbuf,
-                         x, y,
-                         src_width,
-                         src_height,
+                         area_x, area_y,
+                         area_width,
+                         area_height,
                          0, 0,
                          1.0, 1.0,
-                         GDK_INTERP_BILINEAR,
-                         HILDON_HOME_IMAGE_ALPHA_FULL);
-
-    if (src_pixbuf != NULL)
-    {
-        g_object_unref(src_pixbuf);
-    }
-    return;
-}
-
-/**
- * @blend_in_from_path
- *
- * @param pixbuf Storing place of blended image pixbuf
- * @param path Path for the image to read
- * @param save_path Path for the original image area to save
- * @param x Destination for X coordinate
- * @param y Destination for Y coordinate
- *
- * Read an image from given path and blend it into destination pixbuf
- * in the given location.
- */
-static 
-void blend_in_from_path_sidebar(GdkPixbuf **pixbuf, 
-                                const gchar *path, 
-                                const gchar *save_path, 
-                                gint x, gint y)
-{
-    GdkPixbuf *src_pixbuf;
-    GdkPixbuf *src_pixbuf_orig = NULL;
-    GdkPixbuf *pixbuf_sidebar = NULL;
-    gint width, height;
-    gint src_height, src_width;
-    gint loader_status;
-
-    if (*pixbuf == NULL) 
-    {
-        return;
-    }
-
-    width = gdk_pixbuf_get_width(*pixbuf);
-    height = gdk_pixbuf_get_height(*pixbuf);
-
-    if (x > width || y > height) 
-    {
-        return;
-    }
-    
-    loader_status = load_image_pixbuf(&pixbuf_sidebar, path);
-    if(loader_status != HILDON_HOME_IMAGE_LOADER_OK) 
-    {
-        return;
-    }
-
-    src_pixbuf = gdk_pixbuf_scale_simple(
-        pixbuf_sidebar,
-        HILDON_HOME_SIDEBAR_WIDTH_LOADER,
-        HILDON_HOME_SIDEBAR_HEIGHT_LOADER,
-        GDK_INTERP_NEAREST);                
-    if (pixbuf_sidebar != NULL)
-    {
-        g_object_unref(pixbuf_sidebar);
-    }
-
-    src_width = gdk_pixbuf_get_width(src_pixbuf);
-    src_height = gdk_pixbuf_get_height(src_pixbuf);
-
-    if( x+src_width >  width )
-    {
-        src_width = width-x;
-    }
-    if( y+src_height >  height )
-    {
-        src_height = height-y;
-    }
-    
-    loader_status = 
-        create_original_pixbuf_area(&src_pixbuf_orig,
-                                    *pixbuf,
-                                    x, y,
-                                    src_width,
-                                    src_height);
-    if(loader_status != HILDON_HOME_IMAGE_LOADER_OK) 
-    {
-        return;
-    }
-
-    save_image(src_pixbuf_orig, save_path);
-
-    if (src_pixbuf_orig != NULL)
-    {
-        g_object_unref(src_pixbuf_orig);
-    }
-
-    gdk_pixbuf_composite(src_pixbuf, *pixbuf,
-                         x, y,
-                         src_width,
-                         src_height,
-                         0, 0,
-                         1.0, 1.0,
-                         GDK_INTERP_BILINEAR,
+                         GDK_INTERP_NEAREST,
                          HILDON_HOME_IMAGE_ALPHA_FULL);
 
     if (src_pixbuf != NULL)
@@ -716,37 +711,71 @@ void blend_in_from_path_sidebar(GdkPixbuf **pixbuf,
 
 int main(int argc, char *argv[])
 {
-    GdkPixbuf *pixbuf = NULL, *src_pixbuf = NULL;
-    const gchar *operation, *source_uri, *source_uri_save_file;
-    const gchar *dest_filename;
+    gchar *operation;
+    gint scaling;
+    gchar *source_uri, *source_uri_savefile, *target_savefile;
     gint dest_width, dest_height;
-    gint argv_counter = 5, loader_status;
-    gboolean postinstall = FALSE;
+    gint color_red, color_green, color_blue;
 
-    GdkPixbuf *pixbuf_titlebar = NULL;
-    GdkPixbuf *pixbuf_sidebar = NULL;
+    gchar *area1_origfile, *area1_savefile;
+    gint area1_target_x, area1_target_y;
+
+    gchar *area2_origfile, *area2_savefile;
+    gint area2_target_x, area2_target_y;
+
+    gboolean postinstall;
     
-    if (argc == 16 && atoi(argv[15]) == 1)
-    {
-        postinstall = TRUE;
-    }
+    GdkPixbuf *pixbuf = NULL, *src_pixbuf = NULL;
+    GdkPixbuf *pixbuf_area1 = NULL;
+    GdkPixbuf *pixbuf_area2 = NULL;
+    gint loader_status, argc_count;
 
-    if (argc < argv_counter || 
-        (((argc - argv_counter) % 2) != 0 && !postinstall))
+    if (argc != HILDON_HOME_IMAGE_LOADER_ARGC_AMOUNT)
     {
-        g_print("Usage: <operation> <save-image-filename> "
-                "<image-width> <image-height> "
-                "[<source-image-uri> <source-uri-save-file>] "
-                "[<image-filename> <image-filename-save> "
-                "<image-x> <image-y> [..]] [install-flag]\n");
+        g_print("<operation>"
+                " <scaling mode>"
+                " <source-image-uri>"
+                " <source-uri-savefile>"
+                " <image-savefile>"
+                " <image_width>"
+                " <image-heigh>"
+                " <color_red>"
+                " <color_green>"
+                " <color_blue>"
+                " <area1-image-origfile>"
+                " <area1-image-savefile>"
+                " <area1-image-x>"
+                " <area1-image-y>"
+                " <area2-image-origfile>"
+                " <area2-image-savefile>"
+                " <area2-image-x>"
+                " <area2-image-y>"
+                " <install-flag>\n"
+                );
         return HILDON_HOME_IMAGE_LOADER_ERROR_SYSTEM_RESOURCE;
     }
     
-    operation = argv[1];
-    dest_filename = argv[2];
-    dest_width = atoi(argv[3]);
-    dest_height = atoi(argv[4]);
-    
+    argc_count = 1;
+    operation = argv[argc_count++];
+    scaling = atoi(argv[argc_count++]);
+    source_uri = argv[argc_count++];
+    source_uri_savefile = argv[argc_count++];
+    target_savefile = argv[argc_count++];
+    dest_width = atoi(argv[argc_count++]);
+    dest_height = atoi(argv[argc_count++]);
+    color_red = atoi(argv[argc_count++]);
+    color_green = atoi(argv[argc_count++]);
+    color_blue = atoi(argv[argc_count++]);
+    area1_origfile = argv[argc_count++];
+    area1_savefile = argv[argc_count++];
+    area1_target_x = atoi(argv[argc_count++]);
+    area1_target_y = atoi(argv[argc_count++]);
+    area2_origfile = argv[argc_count++];
+    area2_savefile = argv[argc_count++];
+    area2_target_x = atoi(argv[argc_count++]);
+    area2_target_y = atoi(argv[argc_count++]);
+    postinstall = atoi(argv[argc_count++]);
+
     if (!gnome_vfs_init()) 
     {
         return HILDON_HOME_IMAGE_LOADER_ERROR_SYSTEM_RESOURCE;
@@ -754,121 +783,118 @@ int main(int argc, char *argv[])
     
     if(g_str_equal(operation, HILDON_HOME_IMAGE_LOADER_SKIN) == TRUE) 
     {
-        /* -------------------------------- */
-        if(argc >= HILDON_HOME_IMAGE_LOADER_ARGC_TITLEBAR_NEWSKIN) 
+        if(area1_origfile != NULL && area1_savefile != NULL)
         {
-            if(argv[argv_counter] && argv[argv_counter+1])
-            {
-                blend_new_skin_to_image_area(&pixbuf_titlebar,
-                                             argv[argv_counter],
-                                             argv[argv_counter+1]);
-            }
-            /* -------------------------------- */
-            if(argc == HILDON_HOME_IMAGE_LOADER_ARGC_SIDEBAR_NEWSKIN) 
-            {
-                if(argv[argv_counter+4] && argv[argv_counter+5])
-                {
-                    blend_new_skin_to_image_area(&pixbuf_sidebar, 
-                                                 argv[argv_counter+4],
-                                                 argv[argv_counter+5]);
-                }
-            }
-        }        
-        
-        /* -------------------------------- */
-        loader_status = load_image_pixbuf(&pixbuf, dest_filename);
-        if(loader_status != HILDON_HOME_IMAGE_LOADER_OK) 
-        {
-            return loader_status;
-        }
-        
-        
-        /* -------------------------------- */
-        if(argc >= HILDON_HOME_IMAGE_LOADER_ARGC_TITLEBAR_NEWSKIN) 
-        {
-            if(argv[argv_counter] && argv[argv_counter+1] &&
-               pixbuf_titlebar != NULL)
-            {
-                blend_in_with_pixbuf(&pixbuf, pixbuf_titlebar, 
-                                     atoi(argv[argv_counter+2]),
-                                     atoi(argv[argv_counter+3]));
-            }
-        
-            if(argc == HILDON_HOME_IMAGE_LOADER_ARGC_SIDEBAR_NEWSKIN) 
-            {
-                if(argv[argv_counter+4] && argv[argv_counter+5] &&
-                   pixbuf_sidebar != NULL)
-                {
-                    blend_in_with_pixbuf(&pixbuf, pixbuf_sidebar,
-                                         atoi(argv[argv_counter+6]),
-                                         atoi(argv[argv_counter+7]));
-                }
-            }
+            blend_new_skin_to_image_area(&pixbuf_area1,
+                                         area1_origfile,
+                                         area1_savefile);
         }
 
-        loader_status = save_image(pixbuf, dest_filename);
+        if(area2_origfile != NULL && area2_savefile != NULL)
+        {
+            blend_new_skin_to_image_area(&pixbuf_area2,
+                                         area2_origfile,
+                                         area2_savefile);
+        }
+        
+        loader_status = load_image_pixbuf(&pixbuf, target_savefile);
         if(loader_status != HILDON_HOME_IMAGE_LOADER_OK) 
         {
             return loader_status;
         }
-    } else 
+        
+        
+        if(pixbuf_area1 != NULL)
+        {
+            blend_in_with_pixbuf(&pixbuf, pixbuf_area1, 
+                                 area1_target_x,
+                                 area1_target_y);
+        }
+        
+        if(pixbuf_area2 != NULL)
+        {
+            blend_in_with_pixbuf(&pixbuf, pixbuf_area2,
+                                 area2_target_x,
+                                 area2_target_y);
+        }
+
+        loader_status = save_image(pixbuf, target_savefile);
+        if(loader_status != HILDON_HOME_IMAGE_LOADER_OK) 
+        {
+            return loader_status;
+        }
+    } else if(g_str_equal(operation, HILDON_HOME_IMAGE_LOADER_IMAGE) == TRUE)
+
     {
-        source_uri = argv[argv_counter];
-        argv_counter++;
-        source_uri_save_file = argv[argv_counter];
-        argv_counter++;
-        loader_status = 
-            load_image_from_uri(&src_pixbuf, source_uri, postinstall);
+        if(g_str_equal(source_uri, HILDON_HOME_IMAGE_LOADER_NO_IMAGE) == TRUE)
+        {
+            create_background_pixbuf_color(&pixbuf, 
+                                           dest_width, dest_height,
+                                           color_red, color_green, color_blue);
+            if (pixbuf == NULL) 
+            {
+                return HILDON_HOME_IMAGE_LOADER_ERROR_SYSTEM_RESOURCE;
+            }
+        } else
+        {
+            loader_status = 
+                load_image_from_uri(&src_pixbuf, source_uri, postinstall);
         
+            if(loader_status != HILDON_HOME_IMAGE_LOADER_OK) 
+            {
+                return loader_status;
+            }
+            
+            loader_status = create_background_pixbuf(&pixbuf, src_pixbuf, 
+                                                     dest_width, 
+                                                     dest_height,
+                                                     scaling,
+                                                     color_red, 
+                                                     color_green, 
+                                                     color_blue);
+            if (loader_status != HILDON_HOME_IMAGE_LOADER_OK) 
+            {
+                return loader_status;
+            }
+
+            if (src_pixbuf != NULL)
+            {
+                g_object_unref(src_pixbuf);
+            }
+        }
+        /* titlebar area */
+
+        if(area1_origfile != NULL && area1_savefile != NULL)
+        {
+            blend_in_from_path(&pixbuf, 
+                               area1_origfile,
+                               area1_savefile,
+                               area1_target_x,
+                               area1_target_y,
+                               dest_width,
+                               HILDON_HOME_TITLEBAR_HEIGHT_LOADER);
+        }
+
+        /* sidebar area */
+        if(area2_origfile != NULL && area2_savefile != NULL)
+        {
+            blend_in_from_path(&pixbuf, 
+                               area2_origfile,
+                               area2_savefile,
+                               area2_target_x,
+                               area2_target_y,
+                               HILDON_HOME_SIDEBAR_WIDTH_LOADER,
+                               HILDON_HOME_SIDEBAR_HEIGHT_LOADER);
+        }
+        loader_status = save_image(pixbuf, target_savefile);
         if(loader_status != HILDON_HOME_IMAGE_LOADER_OK) 
         {
             return loader_status;
         }
- 
-        loader_status = create_background_pixbuf(&pixbuf, src_pixbuf, 
-                                                 dest_width, dest_height);
-        if (loader_status != HILDON_HOME_IMAGE_LOADER_OK) 
-        {
-            return loader_status;
-        }
-
-        if (src_pixbuf != NULL)
-        {
-            g_object_unref(src_pixbuf);
-        }
-        /* titlebar */
-        if(argc >= HILDON_HOME_IMAGE_LOADER_ARGC_TITLEBAR_NEWIMAGE) 
-        {
-            if(argv[argv_counter] && argv[argv_counter+1])
-            {
-                blend_in_from_path(&pixbuf, 
-                                   argv[argv_counter], 
-                                   argv[argv_counter+1], 
-                                   atoi(argv[argv_counter+2]),
-                                   atoi(argv[argv_counter+3]));
-            }
-        
-            /* sidebar */
-            if(argc == HILDON_HOME_IMAGE_LOADER_ARGC_SIDEBAR_NEWIMAGE) 
-            {
-                argv_counter += 4;
-                if(argv[argv_counter] && argv[argv_counter+1])
-                {
-                    blend_in_from_path_sidebar(&pixbuf, 
-                                               argv[argv_counter], 
-                                               argv[argv_counter+1], 
-                                               atoi(argv[argv_counter+2]),
-                                               atoi(argv[argv_counter+3]));
-                }
-            }
-        }
-
-        loader_status = save_image(pixbuf, dest_filename);
-        if(loader_status != HILDON_HOME_IMAGE_LOADER_OK) 
-        {
-            return loader_status;
-        }
-        save_original_bg_image_uri(source_uri, source_uri_save_file);
+        save_original_bg_image_uri(source_uri, source_uri_savefile);
+    } else
+    {
+        g_print("Unknown operation\n");
     }
         
     if(pixbuf != NULL)
@@ -876,7 +902,5 @@ int main(int argc, char *argv[])
         g_object_unref(pixbuf);
     }
 
-    gnome_vfs_shutdown();
-
     return HILDON_HOME_IMAGE_LOADER_OK;
 }
