@@ -31,7 +31,7 @@
 /* Hildon includes */
 #include "hildon-status-bar-lib.h"
 #include "hildon-status-bar-item.h"
-#include <hildon-widgets/gtk-infoprint.h>
+#include <hildon-widgets/hildon-banner.h>
 #include <hildon-widgets/hildon-note.h>
 
 /* System includes */
@@ -63,8 +63,6 @@
 #include "hildon-status-bar-interface.h"
 #include "../kstrace.h"
 
-gboolean IS_SDK=FALSE;
-
 static gint _delayed_infobanner_add(gint32 pid, gint32 begin, gint32 timeout,
 				    const gchar *text );
 gboolean _delayed_infobanner_remove(gpointer data);
@@ -84,18 +82,21 @@ static gint rpc_cb( const gchar *interface,
                     osso_rpc_t *retval );
 
 static HildonStatusBarItem *get_item( StatusBar *panel, const gchar *plugin );
-static HildonStatusBarItem *add_item( StatusBar *panel, const gchar *plugin );
+static HildonStatusBarItem *add_item( StatusBar *panel, const gchar *path,
+                                                        const gchar *name );
+
+static void statusbar_insensitive_cb( GtkWidget *widget, gpointer data);
+static void statusbar_sensitive_cb( GtkWidget *widget, gpointer data);
 
 static void free_slot( StatusBar *panel );
 static void reorder_items( StatusBar *panel );
 static void destroy_item( GtkObject *object,
                           gpointer user_data );
 
+static gboolean sb_is_sensitive;
 static GHashTable *delayed_banners;
 static gpointer delayed_ib_onscreen;
 
-
-
 static void init_dock( StatusBar *panel )
 {
     Atom atoms[3];
@@ -103,7 +104,7 @@ static void init_dock( StatusBar *panel )
     Display   *dpy;
     Window    win;
 
-    for( i = 0; i < HILDON_STATUS_BAR_MAX_NO_OF_ITEMS; ++i )
+    for( i = 0; i < HSB_MAX_NO_OF_ITEMS; ++i )
         panel->items[i] = NULL;
 
     panel->window = gtk_window_new( GTK_WINDOW_TOPLEVEL );
@@ -145,82 +146,143 @@ static void init_dock( StatusBar *panel )
 
     gtk_container_add( GTK_CONTAINER( panel->window ), panel->fixed );
 
-    panel->plugin_pos_x[0] = HILDON_STATUS_BAR_ITEM0_X;
-    panel->plugin_pos_x[1] = HILDON_STATUS_BAR_ITEM1_X;
-    panel->plugin_pos_x[2] = HILDON_STATUS_BAR_ITEM2_X;
-    panel->plugin_pos_x[3] = HILDON_STATUS_BAR_ITEM3_X;
-    panel->plugin_pos_x[4] = HILDON_STATUS_BAR_ITEM4_X;
-    panel->plugin_pos_x[5] = HILDON_STATUS_BAR_ITEM5_X;
-    panel->plugin_pos_x[6] = HILDON_STATUS_BAR_ITEM6_X;
+    panel->plugin_pos_x[0] = HSB_ITEM0_X;
+    panel->plugin_pos_x[1] = HSB_ITEM1_X;
+    panel->plugin_pos_x[2] = HSB_ITEM2_X;
+    panel->plugin_pos_x[3] = HSB_ITEM3_X;
+    panel->plugin_pos_x[4] = HSB_ITEM4_X;
+    panel->plugin_pos_x[5] = HSB_ITEM5_X;
+    panel->plugin_pos_x[6] = HSB_ITEM6_X;
 
-    panel->plugin_pos_y[0] = HILDON_STATUS_BAR_ITEM0_Y;
-    panel->plugin_pos_y[1] = HILDON_STATUS_BAR_ITEM1_Y;
-    panel->plugin_pos_y[2] = HILDON_STATUS_BAR_ITEM2_Y;
-    panel->plugin_pos_y[3] = HILDON_STATUS_BAR_ITEM3_Y;
-    panel->plugin_pos_y[4] = HILDON_STATUS_BAR_ITEM4_Y;
-    panel->plugin_pos_y[5] = HILDON_STATUS_BAR_ITEM5_Y;
-    panel->plugin_pos_y[6] = HILDON_STATUS_BAR_ITEM6_Y;
+    panel->plugin_pos_y[0] = HSB_ITEM0_Y;
+    panel->plugin_pos_y[1] = HSB_ITEM1_Y;
+    panel->plugin_pos_y[2] = HSB_ITEM2_Y;
+    panel->plugin_pos_y[3] = HSB_ITEM3_Y;
+    panel->plugin_pos_y[4] = HSB_ITEM4_Y;
+    panel->plugin_pos_y[5] = HSB_ITEM5_Y;
+    panel->plugin_pos_y[6] = HSB_ITEM6_Y;
 }
 
+
+/* FIXME Remove this mess */
 static void add_prespecified_items( StatusBar *panel )
 {
 
-    if (add_item( panel, "sound" )==NULL)
+    if (add_item( panel, HSB_PLUGIN_PATH"/libsound.so", "sound" )==NULL)
     {
-        if(IS_SDK==FALSE){
-            osso_log( LOG_ERR, "Statusbar item add failed for sound" );
-        }
+        osso_log( LOG_WARNING, "Statusbar item add failed for sound" );
     }
-    if (add_item( panel, "internet" )==NULL)
+    if (add_item( panel, HSB_PLUGIN_PATH"/libinternet.so", "internet" )==NULL)
     {
-        if(IS_SDK==FALSE){
-            osso_log( LOG_ERR, "Statusbar item add failed for internet" );
-        }
+        osso_log( LOG_WARNING, "Statusbar item add failed for internet" );
     }
-    if (add_item( panel, "gateway" ) ==NULL)
+    if (add_item( panel, HSB_PLUGIN_PATH"/libbattery.so", "battery" ) == NULL)
     {
-        if(IS_SDK==FALSE){
-            osso_log( LOG_ERR, "Statusbar item add failed for gateway" );
-        }
+        osso_log( LOG_WARNING, "Statusbar item add failed for battery" );
     }
-    if (add_item( panel, "battery" ) == NULL)
+    if (add_item( panel, HSB_PLUGIN_PATH"/libdisplay.so", "display" ) == NULL)
     {
-        if(IS_SDK==FALSE){
-            osso_log( LOG_ERR, "Statusbar item add failed for battery" );
-        }
+        osso_log( LOG_WARNING, "Statusbar item add failed for presence" );
     }
-    if (add_item( panel, "display" ) == NULL)
+    if (add_item( panel, HSB_PLUGIN_PATH"/libpresence.so", "presence" ) == NULL)
     {
-        if(IS_SDK==FALSE){
-            osso_log( LOG_ERR, "Statusbar item add failed for display" );
-        }
+        osso_log( LOG_WARNING, "Statusbar item add failed for presence" );
     }
+    
 }
 
 static void add_user_items( StatusBar *panel )
 {
     glob_t globbuf;
-    gchar *pattern, *mark_prefix, *mark_suffix, *name;
-    int i;
+    gchar *pattern;
+    guint i;
 
-    pattern = g_strconcat(HILDON_STATUS_BAR_USER_PLUGIN_PATH, "/lib*.so", NULL);
-    glob(pattern, 0, NULL, &globbuf);
+    pattern = g_strconcat( HSB_DESKTOP_ENTRY_DIR, "/*.desktop", NULL );
+    glob( pattern, 0, NULL, &globbuf );
 
-    for (i = 0; i < globbuf.gl_pathc; i++)
+    for( i = 0; i < globbuf.gl_pathc; i++ )
     {
-        mark_prefix = strrchr(globbuf.gl_pathv[i], '/');
-        mark_suffix = strrchr(globbuf.gl_pathv[i], '.');
-        /* Extract name, cut of lib prefix and .so suffix */
-        name = g_strndup(mark_prefix + 4, mark_suffix - mark_prefix - 4);
-
-        if (add_item( panel, name ) == NULL)
+        GKeyFile *key_file = NULL;
+        GError *error = NULL;
+        gchar *path = NULL;
+        gchar *name = NULL;
+        
+        key_file = g_key_file_new();
+           
+        if( ! g_key_file_load_from_file( key_file, globbuf.gl_pathv[i],
+                    G_KEY_FILE_NONE, &error ) )
         {
-            osso_log( LOG_ERR, "Statusbar item add failed for %s", name );
+            osso_log( LOG_WARNING, "Could not load status bar plugin"
+                                   " desktop file %s", globbuf.gl_pathv[i] );
+            g_key_file_free( key_file );
+            g_error_free( error );
+            continue;
         }
+
+        if( ! g_key_file_has_group( key_file, HSB_DESKTOP_ENTRY_GROUP ) )
+        {
+            osso_log( LOG_WARNING, "Status bar plugin desktop file"
+                                   " %s did not have a Desktop group", 
+                                   globbuf.gl_pathv[i] );
+            g_key_file_free( key_file );
+            continue;
+        }
+
+        path = g_key_file_get_string( key_file,
+                                      HSB_DESKTOP_ENTRY_GROUP,
+                                      HSB_DESKTOP_ENTRY_PATH,
+                                      &error );
+
+        if( !path )
+        {
+            osso_log( LOG_WARNING, "Status bar plugin desktop file"
+                                   " %s did not have a X-status-bar-plugin "
+                                   "entry", 
+                                   globbuf.gl_pathv[i] );
+            g_key_file_free( key_file );
+            g_error_free( error );
+            continue;
+        }
+        
+        name = g_key_file_get_string( key_file,
+                                      HSB_DESKTOP_ENTRY_GROUP,
+                                      HSB_DESKTOP_ENTRY_NAME,
+                                      &error );
+
+        if( !name )
+        {
+            osso_log( LOG_WARNING, "Status bar plugin desktop file"
+                                   " %s did not have a Name "
+                                   "entry", 
+                                   globbuf.gl_pathv[i] );
+            g_key_file_free( key_file );
+            g_error_free( error );
+            continue;
+        }
+            
+        g_key_file_free( key_file );
+
+        if( path[0] != '/' )
+        {
+            gchar *relative_path = path;
+            
+            path  = g_build_filename( HSB_PLUGIN_PATH, relative_path, NULL );
+
+            g_free( relative_path );
+        }
+        
+        if( add_item( panel, path, name ) == NULL )
+        {
+            osso_log( LOG_WARNING, "Statusbar item add failed for %s", name );
+        }
+
+        g_free( path );
+        g_free( name );
+        
     }
 
-    globfree(&globbuf);
-    g_free(pattern);
+    globfree( &globbuf );
+    g_free( pattern );
 }
 
 
@@ -228,7 +290,7 @@ static void show_infoprint( const gchar *msg )
 {
     g_return_if_fail( msg );
 
-    gtk_infoprint( NULL, msg );
+    hildon_banner_show_information( NULL, NULL, msg);
 }
 
 static gint rpc_cb( const gchar *interface, 
@@ -246,7 +308,7 @@ static gint rpc_cb( const gchar *interface,
     if( !interface || !method || !arguments || !data )
     {
         retval->type = DBUS_TYPE_STRING;
-        retval->value.s = "NULL parameter";
+        retval->value.s = g_strdup( "NULL parameter" );
         g_warning( "%s\n", retval->value.s );
         return OSSO_ERROR;
     }
@@ -273,11 +335,9 @@ static gint rpc_cb( const gchar *interface,
                     g_strdup_printf("Wrong type param to infoprint (%d)",
                                      val[0]->type );
             }
-            osso_log( LOG_ERR, retval->value.s );
             if(retval->value.s){
-                g_free((gchar*)retval->value.s);
+                osso_log( LOG_ERR, retval->value.s );
             }
-            retval->value.s = "Wrong param type to infoprint";
 
             return OSSO_ERROR;       
         }
@@ -293,9 +353,9 @@ static gint rpc_cb( const gchar *interface,
         {
             retval->type = DBUS_TYPE_STRING;
             if( arguments->len < 2 ) {
-                retval->value.s = "Not enough args to dialog";
+                retval->value.s = g_strdup( "Not enough args to dialog" );
             } else {
-                retval->value.s = "Wrong type of arguments to dialog";
+                retval->value.s = g_strdup( "Wrong type of arguments to dialog" );
             }
             osso_log( LOG_ERR, retval->value.s );
             return OSSO_ERROR;       
@@ -304,6 +364,79 @@ static gint rpc_cb( const gchar *interface,
         hildon_status_bar_lib_prepare_dialog( val[1]->value.i, NULL, 
                                               val[0]->value.s, 0, NULL, NULL );
     }
+    /* open closeable system dialog */
+    else if( g_str_equal( "open_closeable_system_dialog", method ) )
+    {
+        gint id;
+        const gchar *btext = NULL;
+        /* check arguments */
+        if( arguments->len < 4 ||
+            val[0]->type != DBUS_TYPE_STRING || 
+            val[1]->type != DBUS_TYPE_INT32 ||
+            val[2]->type != DBUS_TYPE_STRING ||
+            val[3]->type != DBUS_TYPE_BOOLEAN ) 
+        {
+            retval->type = DBUS_TYPE_STRING;
+            if( arguments->len < 4 ) {
+                retval->value.s = g_strdup( "Not enough args to dialog" );
+            } else {
+                retval->value.s = g_strdup( "Wrong type of arguments to dialog" );
+            }
+            osso_log( LOG_ERR, retval->value.s );
+            return OSSO_ERROR;       
+        }
+        
+        if( (val[2]->value.s)[0] != '\0' )
+        {
+            btext = val[2]->value.s;
+        }
+
+        id = hildon_status_bar_lib_open_closeable_dialog( val[1]->value.i,
+                 val[0]->value.s, btext, val[0]->value.b );
+        /* return id of the dialog to the caller */
+        retval->type = DBUS_TYPE_INT32;
+        retval->value.i = id;
+    }
+    /* close closeable system dialog */
+    else if( g_str_equal( "close_closeable_system_dialog", method ) )
+    {
+        /* the id of the dialog is given as argument */
+        if( arguments->len < 1 || val[0]->type != DBUS_TYPE_INT32 ) 
+        {
+            retval->type = DBUS_TYPE_STRING;
+            if( arguments->len < 1 ) {
+                retval->value.s = g_strdup( "Not enough args to dialog" );
+            } else {
+                retval->value.s = g_strdup( "Argument has invalid type" );
+            }
+            osso_log( LOG_ERR, retval->value.s );
+            return OSSO_ERROR;       
+        }
+
+        hildon_status_bar_lib_close_closeable_dialog( val[0]->value.i );
+    }
+    /* get response value of system dialog */
+    else if( g_str_equal( "get_system_dialog_response", method ) )
+    {
+        gint response = -1;
+        /* the id of the dialog is given as argument */
+        if( arguments->len < 1 || val[0]->type != DBUS_TYPE_INT32 ) 
+        {
+            retval->type = DBUS_TYPE_STRING;
+            if( arguments->len < 1 ) {
+                retval->value.s = g_strdup( "Not enough args to dialog" );
+            } else {
+                retval->value.s = g_strdup( "Argument has invalid type" );
+            }
+            osso_log( LOG_ERR, retval->value.s );
+            return OSSO_ERROR;       
+        }
+
+        response = hildon_status_bar_lib_get_dialog_response(
+                        val[0]->value.i );
+        retval->type = DBUS_TYPE_INT32;
+        retval->value.i = response;
+    }
     /* plugin */
     else if( g_str_equal( "event", method ) )
     {
@@ -312,8 +445,7 @@ static gint rpc_cb( const gchar *interface,
             val[0]->type != DBUS_TYPE_STRING ||
             val[1]->type != DBUS_TYPE_INT32 ||
             val[2]->type != DBUS_TYPE_INT32 ||
-            ( val[3]->type != DBUS_TYPE_STRING && 
-              val[3]->type != DBUS_TYPE_NIL ) )
+            val[3]->type != DBUS_TYPE_STRING )
         {
             retval->type = DBUS_TYPE_STRING;
             if( arguments->len < 4 )
@@ -322,24 +454,29 @@ static gint rpc_cb( const gchar *interface,
             } else {
                 retval->value.s = g_strdup_printf("Wrong type of arguments to a plugin "
                                      "(%d,%d,%d,%d); "
-                                     "was expecting (%d, %d, %d ,%d or %d)",
+                                     "was expecting (%d, %d, %d ,%d)",
                                      val[0]->type, val[1]->type, val[2]->type, 
                                      val[3]->type,
                                      DBUS_TYPE_STRING, DBUS_TYPE_INT32,
-                                     DBUS_TYPE_INT32, DBUS_TYPE_STRING,
-                                     DBUS_TYPE_NIL);
+                                     DBUS_TYPE_INT32, DBUS_TYPE_STRING);
+                                     //DBUS_TYPE_NIL);
             }
             osso_log( LOG_ERR, retval->value.s );
-            if(retval->value.s){
-                g_free((gchar*)retval->value.s);
-            }
-            retval->value.s = "Wrong type of args to plugin";
             return OSSO_ERROR;
         }
 
         /* Check if the method item is not already loaded */
         if( !(item = get_item( panel, val[0]->value.s ) ) )
-            item = add_item( panel, val[0]->value.s );
+        {
+            /* FIXME lookup the correct path from the .desktop files */
+            gchar *path = g_strconcat( HSB_PLUGIN_PATH, "/lib", val[0]->value.s,
+                                       ".so", NULL );
+            if( path )
+            {
+                item = add_item( panel, path, val[0]->value.s );
+                g_free( path );
+            }
+        }
         
         /* plugin loading failed */
         if( !item )
@@ -347,11 +484,9 @@ static gint rpc_cb( const gchar *interface,
             retval->type = DBUS_TYPE_STRING;
             retval->value.s = g_strdup_printf( "Failed to load plugin %s", 
                                                val[0]->value.s );
-            osso_log( LOG_ERR, retval->value.s );
             if(retval->value.s){
-                g_free((gchar*)retval->value.s);
+                osso_log( LOG_ERR, retval->value.s );
             }
-            retval->value.s="Failed to load plugin";
             return OSSO_ERROR;
         }
 
@@ -367,12 +502,12 @@ static gint rpc_cb( const gchar *interface,
             val[0]->type != DBUS_TYPE_INT32 ||
             val[1]->type != DBUS_TYPE_INT32 ||
             val[2]->type != DBUS_TYPE_INT32 ||
-            val[3]->type != DBUS_TYPE_STRING || 
-	    (arguments->len == 5 && val[5]->type != DBUS_TYPE_NIL 
-	    ) )
-	    {
+            val[3]->type != DBUS_TYPE_STRING ) 
+/*	    (arguments->len == 5 && val[5]->type != DBUS_TYPE_NIL 
+	    ) )*/
+	  {
             retval->type = DBUS_TYPE_STRING;
-            if( arguments->len < 5 ) {
+            if( arguments->len < 4 ) {
                 retval->value.s = 
                     g_strdup_printf( "Not enough arguments." );
             } else {
@@ -386,15 +521,13 @@ static gint rpc_cb( const gchar *interface,
                                      DBUS_TYPE_INT32, 
                                      DBUS_TYPE_STRING);
             }
-            osso_log( LOG_ERR, retval->value.s );
             if(retval->value.s){
-                g_free((gchar*)retval->value.s);
+                osso_log( LOG_ERR, retval->value.s );
             }
-            retval->value.s = "Wrong type of args";
             return OSSO_ERROR;
  
-	    }	
-	    return _delayed_infobanner_add(val[0]->value.i,
+	  }	
+	return _delayed_infobanner_add(val[0]->value.i,
 				       val[1]->value.i,
 				       val[2]->value.i,
 				       val[3]->value.s
@@ -404,7 +537,7 @@ static gint rpc_cb( const gchar *interface,
     {
         if( arguments->len > 1 ||
             val[0]->type != DBUS_TYPE_INT32)
-	    {
+	  {
             retval->type = DBUS_TYPE_STRING;
             if( arguments->len > 1 ) {
                 retval->value.s = g_strdup_printf( "Too many args." );
@@ -416,30 +549,45 @@ static gint rpc_cb( const gchar *interface,
                                      val[0]->type, 
                                      DBUS_TYPE_INT32);
             }
-            osso_log( LOG_ERR, retval->value.s );
             if(retval->value.s){
-                g_free((gchar*)retval->value.s);
+                osso_log( LOG_ERR, retval->value.s );
             }
-            retval->value.s = "Wrong type args";
             return OSSO_ERROR;
 	      
-	    }	
+	  }	
 
-	    _delayed_infobanner_remove(GINT_TO_POINTER(val[0]->value.i));
-	    /* this function returns boolean for the timeout functions, *
-	    * we don't care about that here. It's false always, anyway */
+	_delayed_infobanner_remove(GINT_TO_POINTER(val[0]->value.i));
+	/* this function returns boolean for the timeout functions, *
+	 * we don't care about that here. It's false always, anyway */
         return OSSO_OK;
 
     }
+    else if( g_str_equal( "statusbar_insensitive", method ) )
+    {
+	sb_is_sensitive = FALSE;	
+	gtk_container_foreach(GTK_CONTAINER(panel->fixed), 
+			      (GtkCallback)(statusbar_insensitive_cb),
+			      NULL);
+	    
+	
+	return OSSO_OK;
+    }
+    else if( g_str_equal( "statusbar_sensitive", method ) )
+    {
+	sb_is_sensitive = TRUE;	
+	gtk_container_foreach(GTK_CONTAINER(panel->fixed), 
+			      (GtkCallback)(statusbar_sensitive_cb),
+			      NULL);
+	
+	return OSSO_OK;
+    }
     else
     {
         retval->type = DBUS_TYPE_STRING;
         retval->value.s = g_strdup_printf( "Unknown method '%s'", method );
-        osso_log( LOG_ERR, retval->value.s );
         if(retval->value.s){
-            g_free((gchar*)retval->value.s);
+            osso_log( LOG_ERR, retval->value.s );
         }
-        retval->value.s = "Unknown method";
         return OSSO_ERROR;
     }
 
@@ -454,7 +602,7 @@ static HildonStatusBarItem *get_item( StatusBar *panel, const gchar *plugin )
     g_return_val_if_fail( plugin, NULL );
 
     /* Go through the array and check if the plugin match */
-    for( i = 0; i < HILDON_STATUS_BAR_MAX_NO_OF_ITEMS; ++i )
+    for( i = 0; i < HSB_MAX_NO_OF_ITEMS; ++i )
     {
         if( panel->items[i] != NULL &&
             g_str_equal( hildon_status_bar_item_get_name(
@@ -467,27 +615,31 @@ static HildonStatusBarItem *get_item( StatusBar *panel, const gchar *plugin )
     return NULL;
 }
 
-static HildonStatusBarItem *add_item( StatusBar *panel, const gchar *plugin )
+static HildonStatusBarItem *add_item( StatusBar *panel, const gchar *path,
+                                                        const gchar *name )
 {
     HildonStatusBarItem *item;
     gint slot = -1;
 
     g_return_val_if_fail( panel, NULL );
-    g_return_val_if_fail( plugin, NULL );
+    g_return_val_if_fail( path, NULL );
+    g_return_val_if_fail( name, NULL );
 
     /* check for prespecified items */
-    if( g_str_equal( "sound", plugin ) )
-        slot = HILDON_STATUS_BAR_SOUND_SLOT;
-    else if( g_str_equal( "internet", plugin ) )
-        slot = HILDON_STATUS_BAR_INTERNET_SLOT;
-    else if( g_str_equal( "gateway", plugin ) )
-        slot = HILDON_STATUS_BAR_GATEWAY_SLOT;
-    else if( g_str_equal( "battery", plugin ) )
-        slot = HILDON_STATUS_BAR_BATTERY_SLOT;
-    else if( g_str_equal( "display", plugin ) )
-        slot = HILDON_STATUS_BAR_DISPLAY_SLOT;
+    if( g_str_equal( "sound", name ) )
+        slot = HSB_SOUND_SLOT;
+    else if( g_str_equal( "internet", name ) )
+        slot = HSB_INTERNET_SLOT;
+    else if( g_str_equal( "battery", name ) )
+        slot = HSB_BATTERY_SLOT;
+    else if( g_str_equal( "display", name ) )
+        slot = HSB_DISPLAY_SLOT;
+    else if( g_str_equal( "display", name ) )
+        slot = HSB_DISPLAY_SLOT;
+    else if( g_str_equal( "presence", name ) )
+        slot = HSB_PRESENCE_SLOT;
 
-    item = hildon_status_bar_item_new( plugin );
+    item = hildon_status_bar_item_new( path, name );
 
     if( !item )
     {
@@ -518,11 +670,11 @@ static HildonStatusBarItem *add_item( StatusBar *panel, const gchar *plugin )
         /* free the first slot and put the new item there */
         free_slot( panel );
        
-        panel->items[HILDON_STATUS_BAR_FIRST_DYN_SLOT] = GTK_WIDGET( item );
+        panel->items[HSB_FIRST_DYN_SLOT] = GTK_WIDGET( item );
         gtk_fixed_put( GTK_FIXED( panel->fixed ), 
-                       panel->items[HILDON_STATUS_BAR_FIRST_DYN_SLOT],
-                       panel->plugin_pos_x[HILDON_STATUS_BAR_FIRST_DYN_SLOT],
-                       panel->plugin_pos_y[HILDON_STATUS_BAR_FIRST_DYN_SLOT] );
+                       panel->items[HSB_FIRST_DYN_SLOT],
+                       panel->plugin_pos_x[HSB_FIRST_DYN_SLOT],
+                       panel->plugin_pos_y[HSB_FIRST_DYN_SLOT] );
         gtk_widget_show( GTK_WIDGET( item ) );
 
     }
@@ -535,6 +687,31 @@ static HildonStatusBarItem *add_item( StatusBar *panel, const gchar *plugin )
     return item;
 }
 
+static void statusbar_insensitive_cb( GtkWidget *widget, gpointer data)
+{
+    
+    gtk_widget_set_sensitive(widget, FALSE);
+
+    if (GTK_IS_CONTAINER(widget))
+    {
+	gtk_container_foreach(GTK_CONTAINER(widget), 
+			      (GtkCallback)(statusbar_insensitive_cb),
+			      NULL);	
+    }
+}
+static void statusbar_sensitive_cb( GtkWidget *widget, gpointer data)
+{
+
+    gtk_widget_set_sensitive(widget, TRUE);
+
+    if (GTK_IS_CONTAINER(widget))
+    {
+	gtk_container_foreach(GTK_CONTAINER(widget), 
+			      (GtkCallback)(statusbar_sensitive_cb),
+			      NULL);	
+    }
+}
+
 static void free_slot( StatusBar *panel )
 {
     gint i, j;
@@ -542,37 +719,37 @@ static void free_slot( StatusBar *panel )
     g_return_if_fail( panel );
 
     /* remove last item if too many items */
-    if( panel->items[HILDON_STATUS_BAR_MAX_NO_OF_ITEMS - 1] != NULL )
+    if( panel->items[HSB_MAX_NO_OF_ITEMS - 1] != NULL )
     {
         osso_log( LOG_ERR, "Too many plugins. Defined maximum is %d!", 
-                  HILDON_STATUS_BAR_MAX_NO_OF_ITEMS );
+                  HSB_MAX_NO_OF_ITEMS );
         
-        gtk_widget_destroy(panel->items[HILDON_STATUS_BAR_MAX_NO_OF_ITEMS - 1]);
-        g_object_unref( panel->items[HILDON_STATUS_BAR_MAX_NO_OF_ITEMS - 1]);
-        panel->items[HILDON_STATUS_BAR_MAX_NO_OF_ITEMS - 1] = NULL;
+        gtk_widget_destroy(panel->items[HSB_MAX_NO_OF_ITEMS - 1]);
+        g_object_unref( panel->items[HSB_MAX_NO_OF_ITEMS - 1]);
+        panel->items[HSB_MAX_NO_OF_ITEMS - 1] = NULL;
     }    
         
 
     /* find the last item */
-    for( i = HILDON_STATUS_BAR_MAX_NO_OF_ITEMS - 2; 
-         i >= HILDON_STATUS_BAR_FIRST_DYN_SLOT; --i )
+    for( i = HSB_MAX_NO_OF_ITEMS - 2; 
+         i >= HSB_FIRST_DYN_SLOT; --i )
     {
         if( panel->items[i] == NULL )
             continue;
         
         /* move every item by one */
-        for( j = i; j >= HILDON_STATUS_BAR_FIRST_DYN_SLOT; --j )
+        for( j = i; j >= HSB_FIRST_DYN_SLOT; --j )
         {
             panel->items[j + 1] = panel->items[j];
             panel->items[j] = NULL;
 
             /* remove item from container if moved out of the 7 slots */
-            if( j + 1 == HILDON_STATUS_BAR_VISIBLE_ITEMS )
+            if( j + 1 == HSB_VISIBLE_ITEMS )
                 gtk_container_remove( GTK_CONTAINER( panel->fixed ), 
                                       panel->items[j + 1] );
 
             /* update the fixed if item moved inside the 7 slots */
-            if( j + 1 < HILDON_STATUS_BAR_VISIBLE_ITEMS  )
+            if( j + 1 < HSB_VISIBLE_ITEMS  )
                 gtk_fixed_move( GTK_FIXED( panel->fixed ), panel->items[j + 1],
                                 panel->plugin_pos_x[j + 1],
                                 panel->plugin_pos_y[j + 1] );
@@ -591,11 +768,11 @@ static void reorder_items( StatusBar *panel )
 
     g_return_if_fail( panel );
 
-    place_here = HILDON_STATUS_BAR_FIRST_DYN_SLOT;
+    place_here = HSB_FIRST_DYN_SLOT;
 
     /* go through all dynamic items */
     for( i = place_here; 
-         i < HILDON_STATUS_BAR_MAX_NO_OF_ITEMS; ++i )
+         i < HSB_MAX_NO_OF_ITEMS; ++i )
     {
 
         if (panel->items[i] == NULL) 
@@ -615,14 +792,14 @@ static void reorder_items( StatusBar *panel )
 
         
         /* update the fixed because location changes */
-        if( place_here < HILDON_STATUS_BAR_VISIBLE_ITEMS - 1 )
+        if( place_here < HSB_VISIBLE_ITEMS - 1 )
             gtk_fixed_move( GTK_FIXED( panel->fixed ), 
                             panel->items[place_here],
                             panel->plugin_pos_x[place_here],
                             panel->plugin_pos_y[place_here] );
         
         /* add the plugin to the fixed */
-        else if( place_here == HILDON_STATUS_BAR_VISIBLE_ITEMS - 1)
+        else if( place_here == HSB_VISIBLE_ITEMS - 1)
         {
             gtk_fixed_put( GTK_FIXED( panel->fixed ), 
                            panel->items[place_here],
@@ -651,7 +828,7 @@ static void destroy_item( GtkObject *object,
     item = (GtkWidget *)object;
 
     /* find the item and destroy it */
-    for( i = 0; i < HILDON_STATUS_BAR_MAX_NO_OF_ITEMS; ++i )
+    for( i = 0; i < HSB_MAX_NO_OF_ITEMS; ++i )
     {
         if( panel->items[i] == item )
         {
@@ -727,8 +904,7 @@ static gint _delayed_infobanner_add(gint32 pid, gint32 begin, gint32 timeout,
                         data
                         );
      
-     gtk_banner_show_animation(NULL, 
-                 info->text);
+     info->banner = hildon_banner_show_animation(NULL, NULL, info->text);
                  
  
      return FALSE;
@@ -752,7 +928,8 @@ static gint _delayed_infobanner_add(gint32 pid, gint32 begin, gint32 timeout,
      
      if ( delayed_ib_onscreen == key)
      {
-       gtk_banner_close(NULL);
+       if(data->banner)
+         gtk_widget_destroy(data->banner);
        delayed_ib_onscreen = NULL;
      }
      
@@ -791,7 +968,8 @@ int status_bar_main(osso_context_t *osso, StatusBar **panel){
     gtk_widget_show_all( sb_panel->window );
     TRACE(TDEBUG,"status_bar_main: 7 if rpc...");
     /* set RPC cb */
-    if( osso_rpc_set_default_cb_f( osso, rpc_cb, sb_panel ) != OSSO_OK )
+    if( osso_rpc_set_default_cb_f_with_free( osso, rpc_cb, sb_panel,
+               osso_rpc_free_val ) != OSSO_OK )
     {
         osso_log( LOG_ERR, "osso_rpc_set_default_cb_f() failed" );
     }
@@ -812,7 +990,7 @@ void status_bar_deinitialize(osso_context_t *osso, StatusBar **panel){
   StatusBar *sb_panel = (StatusBar *)(*panel);
 
   TRACE(TDEBUG,"status_bar_deinitialize: 1");
-  for( i = 0; i < HILDON_STATUS_BAR_MAX_NO_OF_ITEMS; ++i )
+  for( i = 0; i < HSB_MAX_NO_OF_ITEMS; ++i )
   {
       if( sb_panel->items[i] != NULL )
       {
@@ -835,7 +1013,6 @@ void status_bar_deinitialize(osso_context_t *osso, StatusBar **panel){
 
 }
 
-
 #ifndef USE_AF_DESKTOP_MAIN__
 
 int main( int argc, char *argv[] )
